#!/usr/bin/perl -w
##################################################
#    yaala 0.3                        09/19/2001 #
#---===========----------------------------------#
# Language: Perl                                 #
# Purpose:  Generating statistics                #
# Input:    Logfiles                             #
# Output:   One or more HTML-files (depending on #
#           the output module used).             #
# Version:  0.3 (stable)                         #
# License:  GPL                                  #
# Homepage: http://13hackerz.de/yaala/           #
# Author:   Florian 'octopus' Forster            #
# Email:    octopus@13hackerz.de                 #
#                                                #
# List of modules (by name):                     #
# --------------------------                     #
# - graph/makegraph.pm                           #
#    Generates the graphs for output/all.pm      #
#    Called by output/all.pm                     #
#                                                #
# - output/all.pm                                #
#    Generates html files with detailed          #
#    statistics using $main::data                #
#    Called by yaala.pl                          #
#    Calls parser/two_dimensional.pm and         #
#    eventually graph/makegraph.pm               #
#                                                #
# - output/top10.pm                              #
#    Generates one html file containing some     #
#    general information from the logfile(s)     #
#    Called by yaala.pl                          #
#    Calls parser/one_dimensional.pm             #
#                                                #
# - parser/Tool.pm                               #
#    Containing some routines used by parsers,   #
#    such as browser and OS recognition.         #
#    Called by parser/apache_access.pm and       #
#    parser/ncsa.pm                              #
#                                                #
# - parser/apache_access.pm                      #
#    Used to parse apache access logfiles.       #
#    Called by yaala.pl                          #
#    Calls parser/Tool.pm                        #
#                                                #
# - parser/ncsa.pm                               #
#    Used to parse NCSA logfiles.                #
#    Called by yaala.pl                          #
#    Calls parser/Tool.pm                        #
#                                                #
# - parser/one_dimensional.pm                    #
#    Provides a routine to add data to the       #
#    internal hash.                              #
#    Called by output/all.pm                     #
#                                                #
# - parser/squid.pm                              #
#    Used to parse squid access logfiles.        #
#    Called by yaala.pl                          #
#                                                #
# - parser/two_dimensional.pm                    #
#    Provides a routine to add data to the       #
#    internal hash.                              #
#    Called by output/top10.pm                   #
##################################################

use strict;

use vars qw( $data $config $VERSION $AUTHOR $EMAIL $HOMEPAGE $debug $extra );

$debug = 1;
$data = {};
$extra = {};

my @files = @ARGV;

if (!@files) { die "You have to declare at least one logfile!\n"; }

unshift (@INC, ('parser', 'output'));

$VERSION = 'yaala 0.3';
$AUTHOR = 'Florian Forster';
$EMAIL = 'octopus@13hackerz.de';
$HOMEPAGE = 'http://13hackerz.de/yaala/';

read_config ();

require $config->{'output'} . '.pm';
require $config->{'logtype'} . '.pm';

foreach (@files)
{
	my $line;
	no strict 'refs';
	open (LOGFILE, '< ' . $_) or die $!;
	while ($line = <LOGFILE>)
	{
		input::parse ($line);
	}
	close LOGFILE;
}
input::extra ();
output::generate ();

exit (0);

##################################################
#    end of main program                         #
#---=====================------------------------#
# surprised?? well, it's pretty short, cause all #
# the _real_ work is done in the the modules.    #
# If you write a modul by your own: PLEASE send  #
# me a copy so that i can include it in the      #
# package.                                       #
##################################################

sub read_config
{
	load_defaults ();
	my $complete_file;
	my %hash;
	%hash = (
		'bar_height'	=>	\$config->{'bar'}{'height'},
		'bar_src0'	=>	\$config->{'bar'}{'src0'},
		'bar_src1'	=>	\$config->{'bar'}{'src1'},
		'bar_width'	=>	\$config->{'bar'}{'width'},
		'bgc_header'	=>	\$config->{'color'}{'header'},
		'bgc_page'	=>	\$config->{'color'}{'bgcolor'},
		'bgc_subhdr'	=>	\$config->{'color'}{'subhdr'},
		'c_links'	=>	\$config->{'color'}{'links'},
		'c_normal'	=>	\$config->{'color'}{'normal'},
		'c_text'	=>	\$config->{'color'}{'text'},
		'draw_graph'	=>	\$config->{'draw_graph'},
		'host_only'	=>	\$config->{'referer'}{'host_only'},
		'logtype'	=>	\$config->{'logtype'},
		'output'	=>	\$config->{'output'},
		'directory'	=>	\$config->{'directory'},
		'ignore'	=>	\$config->{'ignore_less'},
		'include_attacks'=>	\$config->{'inc_attacks'},

		'graph_width'	=>	\$config->{'graph'}{'width'},
		'graph_height'	=>	\$config->{'graph'}{'height'},
		'label_length'	=>	\$config->{'graph'}{'label_length'}
		);
	my %is_array = ('local', 1, 'print', 1);
	local ($/); $/ = undef;
	open (CONFIG, '< config');
	$complete_file = <CONFIG>;
	close (CONFIG);

	$complete_file =~ s/#.*//gm;
	$complete_file =~ s/[\s\t\n\r]+//gs;

	foreach my $pair (split (/;/, $complete_file))
	{
		my ($key, $value) = split (/:/, $pair);
		if (!defined $is_array{$key})
		{
			if (($key =~ /c_/i) and ($value =~ /^[0-9a-fA-F]{6}$/)) { $value = '#' . $value; }
			${$hash{lc ($key)}} = lc $value;
		}
		else
		{
			$config->{lc ($key)}{lc ($value)} = 1;
		}
	}
	die "Please edit the config file first!\nThe included comments should give you a helping hand..\n" if !defined $config->{'print'};
	if ($config->{'logtype'} eq 'apache_combined') { $config->{'logtype'} = 'ncsa'; } #apache_combined is only an alias..
}

sub load_defaults
{
	$config->{'output'}		   = 'all';
	$config->{'logtype'}		   = 'apache_combined';
	$config->{'color'}{'normal'}	   = '#dddddd';
	$config->{'color'}{'header'}	   = '#909090';
	$config->{'color'}{'subhdr'}	   = '#bbbbbb';
	$config->{'color'}{'text'}	   = '#000000';
	$config->{'color'}{'bgcolor'}	   = '#ffffff';
	$config->{'color'}{'links'}	   = '#000088';
	$config->{'bar'}{'height'}	   = 10;
	$config->{'bar'}{'width'}	   = 100;
	$config->{'bar'}{'src0'}	   = 'dot0.gif';
	$config->{'bar'}{'src1'}           = 'dot1.gif';
	$config->{'draw_graph'}		   = 0;
	$config->{'referer'}{'host_only'}  = 1;
	$config->{'graph'}{'width'}        = 600;
	$config->{'graph'}{'height'}       = 250;
	$config->{'graph'}{'label_length'} = 24;
	$config->{'directory'}             = '';
	$config->{'ignore_less'}           = 1;
	$config->{'inc_attacks'}           = 0;
}
